@@ -35,12 +35,12 @@ our $MUNIN_STATEDIR   = q{/var/run/munin};
 our $MUNIN_USER       = q{_munin};
 our $MUNIN_GROUP      = q{_munin};
 our $MUNIN_PLUGINUSER = q{_munin-plugin};
-our $MUNIN_VERSION    = q{2.0.34};
+our $MUNIN_VERSION    = q{2.0.42};
 our $MUNIN_PERL       = q{/usr/bin/perl};
 our $MUNIN_PERLLIB    = q{/usr/local/libdata/perl5/site_perl};
 our $MUNIN_GOODSH     = q{/bin/sh};
 our $MUNIN_BASH       = q{/usr/local/bin/bash};
-our $MUNIN_PYTHON     = q{/usr/local/bin/python2.7};
+our $MUNIN_PYTHON     = q{/usr/bin/env python3};
 our $MUNIN_RUBY       = q{/usr/bin/env ruby};
 our $MUNIN_OSTYPE     = q{openbsd};
 our $MUNIN_HOSTNAME   = q{localhost};
