@@ -2,6 +2,8 @@ package Munin::Master::GraphOld;
 
 # -*- cperl -*-
 
+=encoding utf-8
+
 =begin comment
 
 This is Munin::Master::GraphOld, a package shell to make munin-graph
@@ -11,7 +13,7 @@ feature propper object orientation like munin-update and will have to
 wait until later.
 
 Copyright (C) 2002-2010 Jimmy Olsen, Audun Ytterdal, Kjell Magne
-ierud, Nicolai Langfeldt, Linpro AS, Redpill Linpro AS and others.
+Øierud, Nicolai Langfeldt, Linpro AS, Redpill Linpro AS and others.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -51,7 +53,12 @@ use Time::HiRes;
 use Text::ParseWords;
 
 # For UTF-8 handling (plugins are assumed to use Latin 1)
-if ($RRDs::VERSION >= 1.3) { use Encode; }
+if ($RRDs::VERSION >= 1.3) {
+    use Encode;
+    use Encode::Guess;
+    Encode->import;
+    Encode::Guess->import;
+}
 
 use Munin::Master::Logger;
 use Munin::Master::Utils;
@@ -306,6 +313,8 @@ sub graph_startup {
 	$config = munin_readconfig_part('datafile', 0);
     }
 
+    $config->{debug} = $debug;
+
     my $palette = &munin_get($config, "palette", "default");
 
     $max_running = &munin_get($config, "max_graph_jobs", $max_running);
@@ -1040,6 +1049,10 @@ sub process_service {
             and graph_by_minute($service)) {
             push(@rrd, expand_cdef($service, \$rrdname, "$fname,60,*"));
         }
+        if (    munin_get($field, "type", "GAUGE") ne "GAUGE"
+            and graph_by_hour($service)) {
+            push(@rrd, expand_cdef($service, \$rrdname, "$fname,3600,*"));
+        }
 
         if (my $tmpcdef = munin_get($field, "cdef")) {
             push(@rrd, expand_cdef($service, \$rrdname, $tmpcdef));
@@ -1153,9 +1166,9 @@ sub process_service {
                 unshift(@rrd_negatives,
                     "HRULE:" . $number . ($ldcolour ? "#$ldcolour" : "#$colour"));
             }
-            elsif (my $tmpwarn = munin_get($negfield, "warning",2)) {
+            elsif (my $tmpwarn = munin_get($negfield, "warning")) {
 
-                my ($warn_min, $warn_max) = split(':', $tmpwarn);
+                my ($warn_min, $warn_max) = split(':', $tmpwarn,2);
 
                 if (defined($warn_min) and $warn_min ne '') {
                     unshift(
@@ -1384,8 +1397,8 @@ sub process_service {
         if ($RRDs::VERSION >= 1.3) {
             @complete = map {
                 my $str = $_;
-                $str = encode("utf8", (decode("latin1", $_)));
-                $str;
+                my $utf8 = guess_encoding($str, 'utf8');
+                ref $utf8 ? $str : encode("utf8", (decode("latin1", $_)));
             } @complete;
         }
 
@@ -1493,6 +1506,10 @@ sub process_service {
                                 # Already multiplied by 60
                                 push @replace,
                                     "CDEF:x$fname=PREV($fname),UN,0,PREV($fname),IF,$fname,+,5,*,6,*";
+                            } elsif (graph_by_hour($service)) {
+                                # Already multiplied by 3600, have to *divide* by 12
+                                push @replace,
+                                    "CDEF:x$fname=PREV($fname),UN,0,PREV($fname),IF,$fname,+,12,/,6,*";
                             }
                             else {
                                 push @replace,
@@ -1506,6 +1523,9 @@ sub process_service {
 
                                 # Already multiplied by 60
                                 push @replace, "CDEF:x$fname=$fname,5,*,288,*";
+                            } elsif (graph_by_hour($service)) {
+                                # Already multiplied by 3600, have to *divide* by 12
+                                push @replace, "CDEF:x$fname=$fname,12,/,288,*";
                             }
                             else {
                                 push @replace,
@@ -1576,7 +1596,7 @@ sub handle_trends {
 
     # enddate possibly in future
     my $futuretime = $pinpoint ? 0 : $resolutions{$time} * get_end_offset($service);
-    my $enddate = $lastupdate + ($futuretime);
+    my $enddate = time + ($futuretime);
     DEBUG "[DEBUG] lastupdate: $lastupdate, enddate: $enddate\n";
 
     # future begins at this horizontal ruler
@@ -1600,6 +1620,8 @@ sub handle_trends {
             $colour = "$service->{$fieldname}{'colour'}66";
         }                                                                                                                                                    
                                                                                                                                                              
+        my $rrd_fieldname = get_field_name($fieldname);
+
         my $cdef = "";
         if (defined $service->{$fieldname}{'cdef'}) {
             $cdef = "cdef";
@@ -1607,8 +1629,8 @@ sub handle_trends {
 
         #trends
         if (defined $service->{$fieldname}{'trend'} and $service->{$fieldname}{'trend'} eq 'yes') {
-            push (@complete, "CDEF:t$fieldname=c$cdef$fieldname,$futuretime,TRENDNAN");
-            push (@complete, "LINE1:t$fieldname#$colour:$fieldname trend\\l");
+            push (@complete, "CDEF:t$rrd_fieldname=c$cdef$rrd_fieldname,$futuretime,TRENDNAN");
+            push (@complete, "LINE1:t$rrd_fieldname#$colour:$service->{$fieldname}->{'label'} trend\\l");
             DEBUG "[DEBUG] set trend for $fieldname\n";
         }
 
@@ -1618,8 +1640,8 @@ sub handle_trends {
             my @predict = split(",", $service->{$fieldname}{'predict'});
             my $predictiontime = int ($futuretime / $predict[0]) + 2; #2 needed for 1 day
             my $smooth = $predict[1]*$resolutions{$time};
-            push (@complete, "CDEF:p$fieldname=$predict[0],-$predictiontime,$smooth,c$cdef$fieldname,PREDICT");
-            push (@complete, "LINE1:p$fieldname#$colour:$fieldname prediction\\l");
+            push (@complete, "CDEF:p$rrd_fieldname=$predict[0],-$predictiontime,$smooth,c$cdef$rrd_fieldname,PREDICT");
+            push (@complete, "LINE1:p$rrd_fieldname#$colour:$service->{$fieldname}->{'label'} prediction\\l");
             DEBUG "[DEBUG] set prediction for $fieldname\n";
         }
     }
@@ -1691,6 +1713,12 @@ sub graph_by_minute {
     return (munin_get($service, "graph_period", "second") eq "minute");
 }
 
+sub graph_by_hour {
+    my $service = shift;
+
+    return (munin_get($service, "graph_period", "second") eq "hour");
+}
+
 sub orig_to_cdef {
     my $service   = shift;
     my $fieldname = shift;
@@ -1765,9 +1793,9 @@ sub expand_cdef {
         my $fieldname = munin_get_node_name($field);
         my $rrdname = &orig_to_cdef($service, $fieldname);
         if ($cdef =~ /\b$fieldname\b/) {
-            $max =~ s/([,=])$fieldname([,=]|$)/$1a$rrdname$2/g;
-            $min =~ s/([,=])$fieldname([,=]|$)/$1i$rrdname$2/g;
-            $avg =~ s/([,=])$fieldname([,=]|$)/$1g$rrdname$2/g;
+            $max =~ s/(?<=[,=(])$fieldname(?=[,=)]|$)/a$rrdname/g;                                                                                                                                                                                                    
+            $min =~ s/(?<=[,=(])$fieldname(?=[,=)]|$)/i$rrdname/g;                                                                                                                                                                                                    
+            $avg =~ s/(?<=[,=(])$fieldname(?=[,=)]|$)/g$rrdname/g;                                                                                                                                                                                                    
         }
     }
 
