@@ -94,7 +94,7 @@ sub _do_connect {
 		return 0;
 	}
     } elsif ($uri->scheme eq "ssh") {
-	    my $ssh_command = "ssh -o ChallengeResponseAuthentication=no -o StrictHostKeyChecking=no ";
+	    my $ssh_command = sprintf("%s %s", $config->{ssh_command}, $config->{ssh_options});
 	    my $user_part = ($uri->user) ? ($uri->user . "@") : "";
 	    my $remote_cmd = ($uri->path ne '/') ? $uri->path : "";
 
@@ -146,6 +146,8 @@ sub _do_connect {
 	}
     };
 
+    INFO "[INFO] node $self->{host} advertised itself as $self->{node_name} instead." if $self->{node_name} && $self->{node_name} ne $self->{host};
+
     return 1;
 }
 
@@ -237,15 +239,20 @@ sub list_plugins {
         ? $self->{node_name}
         : $self->{host};
 
-    if (not $host) {
+    my $use_default_node = defined($self->{configref}{use_default_node})
+        ? $self->{configref}{use_default_node}
+        : $config->{use_default_node};
+
+    if (! $use_default_node && ! $host) {
 	die "[ERROR] Couldn't find out which host to list on $host.\n";
     }
 
-    $self->_node_write_single("list $host\n");
+    my $list_host = $use_default_node ? "" : $host;
+    $self->_node_write_single("list $list_host\n");
     my $list = $self->_node_read_single();
 
     if (not $list) {
-        WARN "[WARNING] Config node $self->{host} listed no services for $host.  Please see http://munin-monitoring.org/wiki/FAQ_no_graphs for further information.";
+        WARN "[WARNING] Config node $self->{host} listed no services for $host, (advertised as $self->{node_name}).  Please see http://munin-monitoring.org/wiki/FAQ_no_graphs for further information.";
     }
 
     return split / /, $list;
@@ -276,15 +283,28 @@ sub parse_service_config {
     };
 
 
-    local *push_graphorder = sub {
-	my ($oldservice) = @_;
+   local *push_graphorder = sub {
+		my ($oldservice) = @_;
 
-	push( @{$global_config->{$oldservice}}, 
-	      ['graph_order', join(' ', @graph_order)] )
-	    unless !@graph_order || 
-	           grep { $_->[0] eq 'graph_order' } @{$global_config->{$oldservice}};
-	@graph_order = ( );
-    };
+		# We always appends the field names in config order to any
+		# graph_order given.
+		# Note that this results in duplicates in the internal state
+		# for @graph_order but munin_get_field_order() will eliminate
+		# them before graphing.
+
+		if (@graph_order) {
+			foreach (@{$global_config->{$oldservice}}) {
+				if ( $_->[0] eq 'graph_order' ) {
+					# append to a given graph_order
+					$_->[1] .= join(' ', '', @graph_order);
+					@graph_order = ( );
+					return;
+				}
+			}
+			push @{$global_config->{$oldservice}}, ['graph_order', join(' ', @graph_order)];
+		}
+		@graph_order = ( );
+	};
 
 
     DEBUG "[DEBUG] Now parsing config output from plugin $plugin on "
